#include "stdafx.h"
#include "TutorialApp.h"
#include "SceneTreeControl.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

CSceneTreeControl::CSceneTreeControl( CWnd* pParent )
: CDialog(CSceneTreeControl::IDD, pParent)
, m_PrimEcho( L"" )
, m_CurrentList(NULL)
{
}

void CSceneTreeControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SCENE_TREE, m_SceneTree);
  DDX_Control(pDX, IDC_PRIM_SELECT, m_PrimSelect);
  DDX_Text(pDX, IDC_PRIM_COUNT_ECHO, m_PrimEcho);
}


BEGIN_MESSAGE_MAP(CSceneTreeControl, CDialog)
  ON_NOTIFY(TVN_SELCHANGED, IDC_SCENE_TREE, OnTvnSelchangedSceneTree)
  ON_WM_HSCROLL()
END_MESSAGE_MAP()

// recursive helper function for generating a tree control that
// represents a scene node data structure
static void BuildSceneTree(UWB_SceneNode* pNode, HTREEITEM hParentNode, CTreeCtrl& scenetree)
{
  // recursion ends when there is no child node
  if(NULL == pNode)
    return;

  HTREEITEM hItem = scenetree.InsertItem( pNode->GetName(), hParentNode);
  scenetree.SetItemData(hItem, (DWORD_PTR)pNode);

  for(int i=0; i<pNode->NumChildren(); i++)
  {
    UWB_SceneNode* pChildNode = pNode->GetChildNode(i);
    BuildSceneTree(pChildNode, hItem, scenetree);
  }
}

BOOL CSceneTreeControl::OnInitDialog()
{
  CDialog::OnInitDialog();

  m_PrimSelect.SetRange(0, 0, TRUE);
  m_PrimSelect.SetPos(0);
  m_PrimEcho.Format( L"%d", 0 );
  m_CurrentList = NULL;

  UWBMFC_ReplaceDialogControl(*this, IDC_XFORM_CTRL_PLACEHOLDER, m_XformControl, IDD_XFORMCONTROL_DIALOG);
  m_XformControl.SetSliderRange(-10.0, 10.0f, 0.1f, 3.0f, -180.0f, 180.0f);

  UWBMFC_ReplaceDialogControl(*this, IDC_PRIMITIVE_CONTROL_PLACEHOLDER, m_PrimitiveControl, IDD_PRIMITIVE_CONTROL);

  BuildSceneTree(theApp.GetModel().GetRootNode(), TVI_ROOT, m_SceneTree);
  m_SceneTree.SelectItem(m_SceneTree.GetFirstVisibleItem());

  UpdateData(FALSE);
  return TRUE;  // return TRUE unless you set the focus to a control
}

void CSceneTreeControl::OnTvnSelchangedSceneTree(NMHDR *pNMHDR, LRESULT *pResult)
{
  LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);

  HTREEITEM hItem = pNMTreeView->itemNew.hItem;
  UWB_SceneNode* pNode = reinterpret_cast<UWB_SceneNode*>(m_SceneTree.GetItemData(hItem));

  if(pNode)
  {
    m_XformControl.ShowWindow(SW_SHOW);
    m_PrimitiveControl.ShowWindow(SW_SHOW);

    m_XformControl.SetSceneNode(pNode);
    m_PrimitiveControl.SetPrimitive( pNode->GetPrimitive() );

    theApp.GetModel().SetActiveSceneNode( pNode );
    m_CurrentList = dynamic_cast<UWB_PrimitiveList*>(pNode->GetPrimitive());
    if( m_CurrentList ) 
    {
      m_PrimSelect.SetRange(0, m_CurrentList->Count()-1, TRUE);
      m_PrimSelect.SetPos(0);
      m_PrimEcho.Format( L"%d", 0 );
      UpdateData(FALSE);
      m_PrimitiveControl.SetPrimitive(m_CurrentList->PrimitiveAt( 0 ));
    }
  }
  else
  {
    m_XformControl.ShowWindow(SW_HIDE);
    m_PrimitiveControl.ShowWindow(SW_HIDE);
  }
  *pResult = 0;
}

void CSceneTreeControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  if( m_CurrentList )
  {
    CSliderCtrl *pS = (CSliderCtrl *) pScrollBar;
    int pos = pS->GetPos();
    m_PrimitiveControl.SetPrimitive(m_CurrentList->PrimitiveAt( pos ));
    m_PrimEcho.Format( L"%d", pos );
    UpdateData(FALSE);
  }
  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}
