#include "uwbgl_D3DLight1.h"
#include "uwbgl_D3DGraphicsSystem7.h"


UWBD3D_Light::UWBD3D_Light(int id)
:   m_LightID(id)
  , m_bLightIsOn(false)
  , m_bDrawLight(false)
{
  // Nice set of default values ...
  SetDiffuse(0.8f);
  SetSpecular(0.2f);
  SetAmbient(0.1f);

  SetRange(200.0f);
  SetFalloff(1.0f);
  SetAttenuation0(1.0f);
  SetAttenuation1(0.0f);
  SetAttenuation2(0.0f);
  SetTheta(UWB_PI/4.0f);		// Spot light inner cone radius
  SetPhi(UWB_PI/3.0f);			// Spot light outter cone radius

  SetPosition(0.0f, 0.0f, 0.0f);
  SetDirection(0.0f, -1.0f, 0.0f);
}

UWBD3D_Light::~UWBD3D_Light()
{
}

void UWBD3D_Light::DrawLight(UWB_DrawHelper& draw_helper) {
  if (LightIsSwitchedOn() && ShouldDrawLight() )
    DrawLightGeometry(draw_helper);
}

void UWBD3D_Light::SwitchOnLight()
{
  m_bLightIsOn = true;
}

void UWBD3D_Light::SwitchOffLight()
{
  m_bLightIsOn = false;
}

void UWBD3D_Light::SetLightToDevice() const
{
  UWBD3D_GraphicsSystem& graphics = UWBD3D_GraphicsSystem::GetSystem();
  LPDIRECT3DDEVICE9 device = graphics.GetD3DDevice();
  if(NULL == device || m_LightID == -1)
    return;

  if ( LightIsSwitchedOn() ) {
      device->SetLight(m_LightID, &m_d3dLight);
      device->LightEnable(m_LightID,TRUE);
  } else {
      device->LightEnable(m_LightID,FALSE);
  }
}


void UWBD3D_Light::SetPosition(float x, float y, float z)
{
  m_d3dLight.Position.x = x;
  m_d3dLight.Position.y = y;
  m_d3dLight.Position.z = z;
}

void UWBD3D_Light::SetPosition(const vec3& p)
{
  m_d3dLight.Position.x = p.x;
  m_d3dLight.Position.y = p.y;
  m_d3dLight.Position.z = p.z;
}

void UWBD3D_Light::GetPosition(vec3& p) const
{
  p.x = m_d3dLight.Position.x;
  p.y = m_d3dLight.Position.y;
  p.z = m_d3dLight.Position.z;
}


void UWBD3D_Light::SetDirection(float x, float y, float z)
{
  m_d3dLight.Direction.x = x;
  m_d3dLight.Direction.y = y;
  m_d3dLight.Direction.z = z;
}

void UWBD3D_Light::SetDirection(const vec3& d)
{
  m_d3dLight.Direction.x = d.x;
  m_d3dLight.Direction.y = d.y;
  m_d3dLight.Direction.z = d.z;
}

void UWBD3D_Light::GetDirection(vec3& d) const
{
  d.x = m_d3dLight.Direction.x;
  d.y = m_d3dLight.Direction.y;
  d.z = m_d3dLight.Direction.z;
}