// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_D3DDefines.h"
#include "uwbgl_D3DPrimitiveMesh3.h"
#include <string>

struct DeviceVertexFormat  {
    D3DXVECTOR3  m_Point;   // x,y,z position
    D3DXVECTOR3  m_Normal;   // normal vector at vertex position
    D3DCOLOR     m_Color;    // color for vertex
    FLOAT        m_U,  m_V;  // stage 0 texture coordinates
};


#define DeviceVertexFormatCode (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE|D3DFVF_TEX1)

//
//
// 
class UWBD3D_PrimitiveMeshCustom : public UWBD3D_PrimitiveMesh {


  protected:
    static const int kInvalidVertexIndex;

    bool    m_Initialized;
    bool    m_DrawNormal;
    bool    m_DrawControlGrid;


    /// Mesh initialization happens lazily during run-time, This function
    /// will be called to compute the mesh. Subclass can override this function
    /// to receive initialization call.
    virtual bool InitializeMesh();

    //
    //  D3DXMESH: in our case: caching of MeshResource pointer, we are going to refer to
    //  this structure constantly.
    //
    LPD3DXMESH    	m_Mesh;

    //
    // NumControlVertices is num on each size, e.g. input n means we have a nxn mesh
    //
    // NumVertices = NumControlVertices * NumControlVertices
    // NumFaces = (NumControlVertices - 1)^2 * 2  // (numControlVertices-1) squared then multiply 2
    //
    int    m_NumControlVertices;
    int    m_NumFaces;
    int    m_NumVertices;

    int      		 m_SelectedVertexIndex;
    D3DXVECTOR3  m_SelectedVertex;    // this is only valid when SelectedVertex is valid
                      								// when valid, this value is copied to the buffer each time we draw


    float  m_VertexDistance;

    float ControlVertexSize() const {
            		return VertexDistance() * 0.2f;
          }


    int  IndexFromRowColumn(int r, int c) const {
            return r * m_NumControlVertices + c;
    }

    void LoadSelectedVertexFromD3D();
    void SetSelectedVertexToD3D();

    //
    // 
    virtual void ComputeSurface();

    // override primitive ...
    void DrawPrimitive(eLevelOfDetail lod, UWB_DrawHelper& dh);
    void  DrawMeshControlGrid(eLevelOfDetail lod, UWB_DrawHelper& dh);
    void  DrawNormalVectors(eLevelOfDetail lod, UWB_DrawHelper& dh);

  public:

    int   Resolution() const { return m_NumControlVertices; }
    float VertexDistance() const { return m_VertexDistance; }

    void  SetToNoSelectedVertex();
    void  SelectVertex(int r, int c);
    void  SelectedVertexPosition(vec3 &p) const;
    void  SetSelectedVertexPosition(const vec3 p);

    bool  DrawNormalIsEnabled() { return m_DrawNormal; }
    void  SetDrawNormalTo(bool r) { m_DrawNormal = r; }
    void  EnableDrawNormal() { m_DrawNormal = true; }
    void  DisableDrawNormal() { m_DrawNormal = false; }

    bool DrawControlGridIsEnabled() { return m_DrawControlGrid; }
    void SetDrawControlGridTo(bool r) { m_DrawControlGrid = r; }
    void EnableDrawControlGrid() { m_DrawControlGrid = true; }
    void DisableDrawControlGrid() { m_DrawControlGrid = false; }

    UWBD3D_PrimitiveMeshCustom(int res=20, const wchar_t *outFileName = NULL);
    ~UWBD3D_PrimitiveMeshCustom();
};