#include "uwbgl_D3DPrimitiveMeshCustom1.h"


//
// Selection of control vertex needs a little care since we
// are caching a copy of the vertex geometry locally in main
// memory, we must make sure the cached copy is consistent with
// the version inside the D3D pipeline ...
//


void UWBD3D_PrimitiveMeshCustom::LoadSelectedVertexFromD3D()
{
  if ((m_Initialized) || (InitializeMesh())) {

	    if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		    void *pVBuffer;
		    m_Mesh->LockVertexBuffer( 0, &pVBuffer );
				    DeviceVertexFormat *pCurrentV = &(((DeviceVertexFormat *)pVBuffer)[m_SelectedVertexIndex]);
				    m_SelectedVertex = pCurrentV->m_Point;
		    m_Mesh->UnlockVertexBuffer();
	    }
  }
}

void UWBD3D_PrimitiveMeshCustom::SetSelectedVertexToD3D()
{
  if ((m_Initialized) || (InitializeMesh())) {

	    if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		    void *pVBuffer;
		    m_Mesh->LockVertexBuffer( 0, &pVBuffer );
				    DeviceVertexFormat *pCurrentV = &(((DeviceVertexFormat *)pVBuffer)[m_SelectedVertexIndex]);
				    pCurrentV->m_Point = m_SelectedVertex;
		    m_Mesh->UnlockVertexBuffer();
	    }
  }
}

void UWBD3D_PrimitiveMeshCustom::SetToNoSelectedVertex() 
{
	//
	// check if we are currently selecting a valid vertex
	if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		// now we should update the index into the hardware ...
		SetSelectedVertexToD3D();
	}
	m_SelectedVertexIndex = kInvalidVertexIndex;
}


void UWBD3D_PrimitiveMeshCustom::SelectVertex(int r, int c)
{
	//
	// 1. make sure we are not currently caching anything ...
	//
	SetSelectedVertexToD3D();

	m_SelectedVertexIndex = IndexFromRowColumn(r, c);

	LoadSelectedVertexFromD3D();
}

void UWBD3D_PrimitiveMeshCustom::SelectedVertexPosition(vec3 &p) const
{
	p.x = m_SelectedVertex.x;
  p.y = m_SelectedVertex.y;
  p.z = m_SelectedVertex.z;
}

void UWBD3D_PrimitiveMeshCustom::SetSelectedVertexPosition(const vec3 p)
{
	m_SelectedVertex.x = p.x;
  m_SelectedVertex.y = p.y;
  m_SelectedVertex.z = p.z;
	//
	// Notice we cache this, and only set into the Hardware before we draw ...
	//
}