// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Primitive6.h"
#include "uwbgl_Containers.h"

/// Notice that UWB_PrimitiveList is a subclass of UWB_Primitive.
/// This allows us to call Draw(...) on this list.
class UWB_PrimitiveList : public UWB_Primitive
{
public:
  UWB_PrimitiveList();

  /// Update the position of the line based in it's velocity
  /// \param elapsed_seconds amount of time that has passed since the last update
  virtual void Update(float elapsed_seconds);

  /// Virtual override for getting the bounding volume of this primitive
  /// the bounding volume of this list is a union of all the child bounding volumes in the list
  virtual const UWB_BoundingVolume* GetBoundingVolume( eLevelOfDetail lod ) const;
  ///
  /// Draw the bounding volumes of all the child primitives in this list
  void DrawChildBoundingVolumes(eLevelOfDetail lod, UWB_DrawHelper* pDrawHelper, UWB_Color color) const;

  ///
  /// Number of child primitives in this list
  int Count() const;

  /// Get primitive at a given index in this list.
  /// Returns NULL if the index is outside of the lists range
  UWB_Primitive* PrimitiveAt(int index);
  /// Get primitive at a given index in this list.
  /// Returns NULL if the index is outside of the lists range
  const UWB_Primitive* PrimitiveAt(int index) const;

  /// Add a primitive to to this list. The primitive must be created on the heap.
  /// This list controls the lifetime of the primitive
  void Append(UWB_Primitive* pPrimitive);

  ///
  /// Delete a primitive at a given index in the list
  void DeletePrimitiveAt(int index);

protected:
  /// Virtual drawing routine for this list. This just calls the draw function
  /// on all of the child primitives in the list
  virtual void DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper );

  /// Virtual function used to define the current attributes for drawing. This version
  /// does nothing because each child primitive will get a call to it's individual SetupDrawAttributes
  virtual void SetupDrawAttributes(UWB_DrawHelper& draw_helper);

protected:
  ///
  /// The list of primitives that this primitive contains
  UWB_PointerArray<UWB_Primitive*> m_list;

  //for the lowest level of detail, use a bounding box that is the union of all child bounding volumes
  UWB_BoundingBox m_bounds;
};
