#include "uwbgl_WindowHandler7.h"
#include "uwbgl_Utility2.h"

UWB_WindowHandler::UWB_WindowHandler()
: m_hAttachedWindow(NULL)
{
}

UWB_WindowHandler::~UWB_WindowHandler()
{
}

bool UWB_WindowHandler::InitializeHandler(HWND hWindow)
{
  //check that the parameter is an actual window
  if(!::IsWindow(hWindow))
    return false;

  m_hAttachedWindow = hWindow;
  return true;
}

bool UWB_WindowHandler::HardwareToDevice(int hardwareX, int hardwareY, int& deviceX, int& deviceY) const
{
  if (!IsWindow(m_hAttachedWindow))
    return false;

  // x coordinate is the same
  deviceX = hardwareX;

  // transform is a flip of Y
  RECT rect;
  GetClientRect(m_hAttachedWindow, &rect);
  int client_height = rect.bottom - rect.top;
  deviceY = client_height - hardwareY;

  return true;
}

float UWB_WindowHandler::GetAspectRatio() const
{
  if (!IsWindow(m_hAttachedWindow))
    return 0.0f;;

  RECT rect;
  GetClientRect(m_hAttachedWindow, &rect);

  if((rect.bottom - rect.top) <= 0)
    return 0.0f;

  return (float)(rect.right - rect.left) / (float)(rect.bottom - rect.top);
}

void UWB_WindowHandler::SetWCWindow(const UWB_BoundingBox &w)
{
	m_WCWindow = w;
}

UWB_BoundingBox UWB_WindowHandler::GetWCWindow() const
{
  return m_WCWindow;
}

void  UWB_WindowHandler::DeviceToWorld(int dcx, int dcy, float &wcx, float &wcy) const 
{ 
  int dcW, dcH;
  GetDeviceSize(dcW, dcH);
	vec3 center = m_WCWindow.GetCenter();

  wcx = center.x + ( (m_WCWindow.Width()/dcW)  * (dcx - (dcW/2.0f)) );
  wcy = center.y + ( (m_WCWindow.Height()/dcH) * (dcy - (dcH/2.0f)) );

	// wcx = wcMin.x + (dcx * (m_WCWindow.Width() / float(dcW)) );
	// wcy = wcMin.y + (dcy * (m_WCWindow.Height() / float(dcH)) );
}

void  UWB_WindowHandler::WorldToDevice(float wcx, float wcy, int &dcx, int &dcy) const 
{ 
  int dcW, dcH;
  GetDeviceSize(dcW, dcH);
	vec3 center = m_WCWindow.GetCenter();

  // dcx = int( ((wcx - wcMin.x) * dcW / m_WCWindow.Width()) + 0.5f );
  // dcy = int( ((wcy - wcMin.y) * dcH / m_WCWindow.Height()) + 0.5f );

  dcx = UWB_Round( (m_WCWindow.Width()  / 2.0f) + ( (dcW/m_WCWindow.Width())  * (-center.x + wcx) ) );
  dcy = UWB_Round( (m_WCWindow.Height() / 2.0f) + ( (dcH/m_WCWindow.Height()) * (-center.y + wcy) ) );
}

void UWB_WindowHandler::SetDeviceSize(int width, int height)
{
  RECT rect;
  HWND parent = GetParent(m_hAttachedWindow);
  GetWindowRect(m_hAttachedWindow, &rect);
  POINT p; 
  p.x = rect.left;
  p.y = rect.top;
  ScreenToClient(parent, &p);
  MoveWindow(m_hAttachedWindow, p.x, p.y, width, height, TRUE);
}

void UWB_WindowHandler::GetDeviceSize(int &width, int &height) const
{
  RECT rect;
  GetClientRect(m_hAttachedWindow, &rect);
	width = rect.right - rect.left;
	height = rect.bottom - rect.top;
}

void UWB_WindowHandler::DrawWCWindow(UWB_DrawHelper &drawHelper) const
{
  drawHelper.ResetAttributes();
  drawHelper.SetShadeMode( smFlat);
  drawHelper.SetFillMode(fmWireframe);
  drawHelper.SetColor1( UWB_Color(1.0f, 0.0f, 0.0f) );
  drawHelper.DrawRectangleXY( m_WCWindow.GetMin(), m_WCWindow.GetMax() );
}

void UWB_WindowHandler::DeviceToWorld(int dcX, int dcY, vec3 &wcPt, vec3 &wcRay) const 
{ 
  wcPt.x = wcPt.y = wcPt.z = 0.0f;
  wcRay.x = wcRay.y = wcRay.z = 0.0f;
  MessageBox(NULL,
             L"UWB_WindowHandler::DeviceToWorld(): is not overrided-OnScreen picking will failed!!", 
             L"UWB_WindowHandler::DeviceToWorld",
             MB_OK);
}

void UWB_WindowHandler::SetCamera(const UWB_Camera &c)
{
  m_Camera = c;
}

UWB_Camera& UWB_WindowHandler::GetCamera() 
{
  return m_Camera;
}
